<?php
/*
Plugin Name: MintPopularPostsWP
Plugin URI: http://turmsegler.net/20070904/wp-plugin-mintpopularposts/
Description: Creates a list of popular posts based on an attached <a href="http://haveamint.com">Mint</a> tracking system. No configuration required. WordPress caching is used if available.
Version: 1.0.1
Author: Benjamin Stein
Author URI: http://turmsegler.net/
*/

/* 

mpp_popular_posts

Parameter 	        Default 	Description
$num_posts   	    10 	        The number of posts to display
$prev_days   	    0 	        How many days would you like the list to run for? For all posts, 
                                enter 0 (default). Any other number (X) will retrieve the top posts
                                in the last X days
$text_to_remove 	 	        The text to remove from each post title that is displayed. For example,
                                if every title begins with  Turmsegler it would be redundant to enter 
                                this as part of the link text for every post listed.
$ignore_list 	     	        comma-seperated list of URLs (including http://) that should not be counted 
                                in the popular posts list (wildcard = * or %)
$ignore_extensions 	 	        comma-seperated list of file extensions to ignore (wildcard = * or %)
$include_count 	    true 	    Should the count for each post be displayed?
$include_www 	    false 	    Should a www be appended to the beginning of every url (changing 
                                http://turmsegler.net into http://www.turmsegler.net)?
$pre 	            <li> 	    the html that should be included before every post listing
$post 	            </li> 	    the html that should be included after every post listing


mpp_watched_posts

Parameter 	        Default 	Description
$num_posts   	    10 	        The number of posts to display
$text_to_remove 	 	        The text to remove from each post title that is displayed. For example,
                                if every title begins with  Turmsegler it would be redundant to enter 
                                this as part of the link text for every post listed.
$include_count 	    true 	    Should the count for each post be displayed?
$include_www 	    false 	    Should a www be appended to the beginning of every url (changing 
                                http://turmsegler.net into http://www.turmsegler.net)?
$pre 	            <li> 	    the html that should be included before every post listing
$post 	            </li> 	    the html that should be included after every post listing

*/

add_option('mpp_mint_db_name'  ,  __(''      , 'mpp') );
add_option('mpp_mint_db_user'  ,  __(''      , 'mpp') );
add_option('mpp_mint_db_pwd'   ,  __(''      , 'mpp') );
add_option('mpp_mint_db_host'  ,  __(''      , 'mpp') );
add_option('mpp_mint_ign_list' ,  __(''      , 'mpp') );
add_option('mpp_mint_ign_ext'  ,  __(''      , 'mpp') );
add_option('mpp_mint_url_pat'  ,  __(''      , 'mpp') );
add_option('mpp_mint_prefix'   ,  __('mint_' , 'mpp') );
add_option('mpp_mint_ign_ext'  ,  __(''      , 'mpp') );
add_option('mpp_mint_db_ext'   ,  __(0       , 'mpp') );
add_option('mpp_mint_title_rem',  __(''      , 'mpp') );


function mpp_watched_posts (
          $num_posts         = 10 
        , $text_to_remove    = '' 
        , $include_count     = true
        , $include_www       = false 
        , $pre               = '<li>' 
        , $post              = '</li>' ) 
{
    mpp_popular_posts_internal (
          $num_posts
        , 0
        , $text_to_remove
        , ''       
        , ''
        , $include_count
        , $include_www
        , $pre
        , $post
        , true );
}

function mpp_popular_posts (
          $num_posts         = 10 
        , $prev_days         = 0 
        , $text_to_remove    = '' 
        , $ignore_list       = '' 
        , $ignore_extensions = ''
        , $include_count     = false
        , $include_www       = false 
        , $pre               = '<li>' 
        , $post              = '</li>' ) 
{
    mpp_popular_posts_internal (
          $num_posts
        , $prev_days
        , $text_to_remove
        , $ignore_list
        , $ignore_extensions
        , $include_count
        , $include_www
        , $pre
        , $post
        , false );
}

function mpp_popular_posts_internal (
          $num_posts         = 10 
        , $prev_days         = 0 
        , $text_to_remove    = '' 
        , $ignore_list       = '' 
        , $ignore_extensions = ''
        , $include_count     = false
        , $include_www       = false 
        , $pre               = '<li>' 
        , $post              = '</li>'
        , $watched           = false ) 
{
    global $wpdb;
    
    /* Get options */
    global $mpp_mint_db_name;
    global $mpp_mint_db_user;
    global $mpp_mint_db_pwd;
    global $mpp_mint_db_host;
    global $mpp_mint_db_ext;
    global $mpp_mint_ign_list;
    global $mpp_mint_ign_ext;
    global $mpp_mint_url_pat;
    global $mpp_mint_prefix;
    global $mpp_mint_title_rem;
    
    $mpp_mint_db_name   = get_option('mpp_mint_db_name');
    $mpp_mint_db_user   = get_option('mpp_mint_db_user');
    $mpp_mint_db_pwd    = get_option('mpp_mint_db_pwd');
    $mpp_mint_db_host   = get_option('mpp_mint_db_host');
    $mpp_mint_db_ext    = get_option('mpp_mint_db_ext');
    $mpp_mint_ign_list  = get_option('mpp_mint_ign_list');
    $mpp_mint_ign_ext   = get_option('mpp_mint_ign_ext');
    $mpp_mint_url_pat   = get_option('mpp_mint_url_pat');
    $mpp_mint_prefix    = get_option('mpp_mint_prefix');
    $mpp_mint_title_rem = get_option('mpp_mint_title_rem');
    

    $parmStr  = '#' . $num_posts;
    $parmStr .= '#' . $text_to_remove;
    $parmStr .= '#' . $mpp_mint_db_name;
    $parmStr .= '#' . $mpp_mint_db_user;
    $parmStr .= '#' . $mpp_mint_db_pwd;
    $parmStr .= '#' . $mpp_mint_db_host;
    $parmStr .= '#' . $mpp_mint_db_ext;
    $parmStr .= '#' . $mpp_mint_title_rem;

    if ( !$watched )
    {
        $parmStr .= '#' . $ignore_list;
        $parmStr .= '#' . $prev_days;
        $parmStr .= '#' . $ignore_extensions;
        $parmStr .= '#' . $mpp_mint_ign_list;
        $parmStr .= '#' . $mpp_mint_ign_ext;
        $parmStr .= '#' . $mpp_mint_url_pat;
        $parmStr .= '#' . $mpp_mint_prefix;
    }
    else
    {
        $parmStr .= '#watched';
    }
    
    if ( $include_count )
        $parmStr .= '#c_true';
    
    if ( $include_www )
        $parmStr .= '#w_true';

    // make a unique WordPress cache key
    $key    = 'mpp_gpp#' . md5( $parmStr );

    // try to fetch the output from WordPress cache
    $result = wp_cache_get($key, 'mpp_gpp');

    if ( false !== $result )
    {
        $result .= '<!-- mpp_gpp_cached -->';
    }

    // if it cannot be read from the cache
    // create the output dynamically

    if ( false === $result )
    {
        $result = mpp_get_popular_posts(
                              $num_posts
                            , $prev_days 
                            , $text_to_remove 
                            , $ignore_list 
                            , $ignore_extensions
                            , $include_count
                            , $include_www
                            , $pre 
                            , $post
                            , $watched ); 

        if ( false !== $result )
            wp_cache_set( $key, $result, 'mpp_gpp' );
    }

    echo $result;
}

function mpp_get_popular_posts (
          $num_posts         = 10 
        , $prev_days         = 0 
        , $text_to_remove    = '' 
        , $ignore_list       = '' 
        , $ignore_extensions = ''
        , $include_count     = false
        , $include_www       = false 
        , $pre               = '<li>' 
        , $post              = '</li>'
        , $watched           = false ) 
{
    /* Get options */
    global $mpp_mint_db_name;
    global $mpp_mint_db_user;
    global $mpp_mint_db_pwd;
    global $mpp_mint_db_host;
    global $mpp_mint_db_ext;
    global $mpp_mint_ign_list;
    global $mpp_mint_ign_ext;
    global $mpp_mint_url_pat;
    global $mpp_mint_prefix;
    global $mpp_mint_title_rem;

    $isIgnore       = false;
    $isPattern      = false;
    $ig_list        = array();
    $ig_ext         = array();
    $referrerCounts = array();
    $pageData       = array();
    $where          = '';
    
    // override some default options
    if ( $ignore_list != '' && $mpp_mint_ign_list != $ignore_list )
        $mpp_mint_ign_list = $ignore_list;
        
    if ( $ignore_extensions != '' && $mpp_mint_ign_ext != $ignore_extensions )
        $mpp_mint_ign_ext = $ignore_extensions;

    if ( $text_to_remove != '' && $mpp_mint_title_rem != $text_to_remove )
        $mpp_mint_title_rem = $text_to_remove;
    
    
    if ($mpp_mint_title_rem != '') 
    {
        $text_to_remove = str_replace(",,","-%$%-",$mpp_mint_title_rem);
        $text_to_remove_list = explode(',',$mpp_mint_title_rem);
    
        for ($x = 0 ; $x < count($text_to_remove_list) ; $x++) 
        {
            $text_to_remove_list[$x] = str_replace("-%$%-",",",$text_to_remove_list[$x]);
            $text_to_remove_list[$x] = trim($text_to_remove_list[$x]);
        }
    }
    
    if ( !$watched && $mpp_mint_ign_list != '') 
    {
        $mpp_mint_ign_list = str_replace(",,","-%$%-",$mpp_mint_ign_list);
        $mpp_mint_ign_list = str_replace(" ","", $mpp_mint_ign_list);  
        $mpp_mint_ign_list = str_replace("*", "%", $mpp_mint_ign_list);
        $ig_list           = explode(',',$mpp_mint_ign_list);
        for ($x = 0 ; $x < count($ig_list) ; $x++) 
        {
            $ig_list[$x] = str_replace("-%$%-",",",$ig_list[$x]);
        }
        $isIgnore = true;
    }

    if ( !$watched && $mpp_mint_ign_ext != '') 
    {
        $mpp_mint_ign_ext = str_replace(" ","", $mpp_mint_ign_ext); 
        $mpp_mint_ign_ext = str_replace("*", "%", $mpp_mint_ign_ext);
        $ig_ext           = explode(',',$mpp_mint_ign_ext);
        for ($j = 0 ; $j < count($ig_ext) ; $j++) 
        {
            $ig_ext[$j] = '%'.$ig_ext[$j];
        }
        $isIgnore = true;
    }
    
    if ( $isIgnore ) 
    {
        $ignore = array_merge( $ig_list, $ig_ext );    
    }

    if ( !$watched && $mpp_mint_url_pat != '') 
    {
        $mpp_mint_url_pat = str_replace(",,","-%$%-",$mpp_mint_url_pat);
        $mpp_mint_url_pat = str_replace(" ","", $mpp_mint_url_pat);  
        $mpp_mint_url_pat = str_replace("*", "%", $mpp_mint_url_pat);
        $pat_list         = explode(',',$mpp_mint_url_pat);
        for ($x = 0 ; $x < count($ig_list) ; $x++) 
        {
            $pat_list[$x] = str_replace("-%$%-",",",$pat_list[$x]);
        }
        $isPattern = true;
    }

    // If is external DB, set DB
    if ($mpp_mint_db_ext && $mpp_mint_db_name != '') 
    {
        $mysql = mysql_connect($mpp_mint_db_host, $mpp_mint_db_user, $mpp_mint_db_pwd);
        
        if ( !$mysql )
        {
            echo 'Could not connect to Mint database: ' . mysql_error();
            return false;
        }
        if ( !mysql_select_db($mpp_mint_db_name) )
        {
            echo 'Could not select Mint database!';
            return false;
        }
    }

    if ( $watched )
    {
        $query = "SELECT data FROM {$mpp_mint_prefix}_config LIMIT 0,1";
    
        if ($result = mysql_query($query))
        {
            if ($load = mysql_fetch_assoc($result))
            {
                $data           = mmp_unserialize($load['data']);
                $DefPepperData  = $data[0];
                $DefPepperWatch = $DefPepperData['watched'];
    
                if ( !empty($DefPepperWatch) && is_array($DefPepperWatch) ) 
                {
                    $where .= " AND ( resource_checksum = '".implode("' OR resource_checksum = '", $DefPepperWatch)."') ";
                }
            }
        }
    }

    // add optional URL pattern
    if ( $isPattern )
        $where .= " AND ( resource LIKE '".implode("' OR resource LIKE '", $pat_list)."') ";
    
    // add all items from ignore string
    if ( $isIgnore ) 
        $where .= " AND resource NOT LIKE '".implode("' AND resource NOT LIKE '", $ignore)."' ";
        
    // Calculate time limit
    if ( $prev_days > 0 ) 
    {
        $time_start = time() - ($prev_days * 24 * 60 * 60);
        $where    .= " AND dt > {$time_start} ";
    }
    
    $query = "SELECT resource_checksum
                   , COUNT(DISTINCT referer) AS referrers
                FROM {$mpp_mint_prefix}visit 
               WHERE referer_is_local = 0
                 AND search_terms     = ''
                     {$where}
            GROUP BY resource_checksum 
            ORDER BY referrers DESC";

    // echo "<!-- {$query} -->";

    if ($result = mysql_query($query)) 
    {
        while ($r = mysql_fetch_assoc($result)) 
        {
            $referrerCounts[$r['resource_checksum']] = $r['referrers'];
        }
    }
    
    $query = "SELECT resource
                   , resource_title
                   , resource_checksum
                   , COUNT(resource_checksum) AS total
                FROM {$mpp_mint_prefix}visit  
               WHERE 1=1
                     {$where}
            GROUP BY resource_checksum 
            ORDER BY total DESC LIMIT {$num_posts}";

    // echo "<!-- {$query} -->";

    if ($result = mysql_query($query)) 
    {
        while ($r = mysql_fetch_assoc($result)) 
        {
            $pageData[$r['resource_checksum']] = $r;
        }
    }

    $result = '';
    
    foreach ($pageData as $checksum => $page)
    {
        $resource       = ($include_www) ? str_replace("http://", "http://www.", $page['resource']) : $page['resource'];
        $resource_cnt   = $page['total'];        
        $resource_ref   = (isset($referrerCounts[$checksum])) ? $referrerCounts[$checksum] : 0;
        $resource_title = trim(stripslashes($page['resource_title']));
        $resource_ext   = ($include_count) ? " ({$resource_cnt})" : "";
        
        for ($x = 0 ; $x < count($text_to_remove_list) ; $x++) 
        {
            $resource_title = str_replace($text_to_remove_list[$x], "", $resource_title);
        }

        $resource_title = trim($resource_title);

        if ( $watched )
        {
            $result .=  "<li><a title=\"{$resource_title} ({$resource_cnt} hits, {$resource_ref} referrers)\" rel=\"bookmark\" ";
            $result .=  "href=\"{$resource}\">{$resource_title}{$resource_ext}</a></li>";
        }
        else
        {
            $result .=  "<li><a title=\"{$resource_title} ({$resource_cnt})\" rel=\"bookmark\" ";
            $result .=  "href=\"{$resource}\">{$resource_title}{$resource_ext}</a></li>";
        }
    }
   
    if ($mpp_mint_db_ext && $mpp_mint_db_name != '') 
    {
        mysql_close($mysql);
        $wpdb = new wpdb(DB_USER, DB_PASSWORD, DB_NAME, DB_HOST);
    }
    
    return $result;
}

function mpp_options_panel()
{
    // check form submission and update options
    if ('process' == $_POST['stage'])
    {
        update_option('mpp_mint_db_name'  , $_POST['mpp_mint_db_name']);
        update_option('mpp_mint_db_user'  , $_POST['mpp_mint_db_user']);
        update_option('mpp_mint_db_pwd'   , $_POST['mpp_mint_db_pwd']);
        update_option('mpp_mint_db_host'  , $_POST['mpp_mint_db_host']);
        update_option('mpp_mint_ign_list' , $_POST['mpp_mint_ign_list']);
        update_option('mpp_mint_ign_ext'  , $_POST['mpp_mint_ign_ext']);
        update_option('mpp_mint_url_pat'  , $_POST['mpp_mint_url_pat']);
        update_option('mpp_mint_prefix'   , $_POST['mpp_mint_prefix']);
        update_option('mpp_mint_title_rem', $_POST['mpp_mint_title_rem']);
    
        if ( isset($_POST['mpp_mint_db_ext']) )
           update_option('mpp_mint_db_ext', 1);
        else
           update_option('mpp_mint_db_ext', 0);
    }
    
    /* Get options */
    $mpp_mint_db_name   = get_option('mpp_mint_db_name');
    $mpp_mint_db_user   = get_option('mpp_mint_db_user');
    $mpp_mint_db_pwd    = get_option('mpp_mint_db_pwd');
    $mpp_mint_db_host   = get_option('mpp_mint_db_host');
    $mpp_mint_db_ext    = get_option('mpp_mint_db_ext');
    $mpp_mint_ign_list  = get_option('mpp_mint_ign_list');
    $mpp_mint_ign_ext   = get_option('mpp_mint_ign_ext');
    $mpp_mint_url_pat   = get_option('mpp_mint_url_pat');
    $mpp_mint_prefix    = get_option('mpp_mint_prefix');
    $mpp_mint_title_rem = get_option('mpp_mint_title_rem');

    if ( $mpp_mint_db_ext )
        $mpp_mint_db_ext_flag = 'checked="checked"';
    else
        $mpp_mint_db_ext_flag = '';

    echo    
    '<div class="wrap">
      <h2>Mint Popular Posts Options</h2>
        <form name="form1" method="post" action="options-general.php?page=MintPopularPostsWP.php&updated=TRUE">
           <input type="hidden" name="stage" value="process" />
           <input type="hidden" name="wprvm_start_day" id="wprvm_start_day" value="'.$wprvm_start_day.'" />
               <table width="100%" cellspacing="2" cellpadding="5" class="editform">
               <tr valign="top">
                   <th scope="row">Use External Mint DB</th>
                   <td>
                   <input name="mpp_mint_db_ext" type="checkbox" id="mpp_mint_db_ext" value="mpp_mint_db_ext" '.$mpp_mint_db_ext_flag.' />
                   </td>
               </tr>
               <tr valign="top">
                    <th scope="row">External Mint DB - Database Name:</th>
                    <td><input name="mpp_mint_db_name" type="text" id="mpp_mint_db_name" value="'.$mpp_mint_db_name.'" size="50" />
               </tr>
               <tr valign="top">
                    <th scope="row">External Mint DB - Database User:</th>
                    <td><input name="mpp_mint_db_user" type="text" id="mpp_mint_db_user" value="'.$mpp_mint_db_user.'" size="50" />
               </tr>
               <tr valign="top">
                    <th scope="row">External Mint DB - Database Password:</th>
                    <td><input name="mpp_mint_db_pwd" type="text" id="mpp_mint_db_pwd" value="'.$mpp_mint_db_pwd.'" size="50" />
               </tr>
               <tr valign="top">
                    <th scope="row">External Mint DB - Database Host:</th>
                    <td><input name="mpp_mint_db_host" type="text" id="mpp_mint_db_host" value="'.$mpp_mint_db_host.'" size="50" />
               </tr>
               <tr valign="top">
                    <th scope="row">Mint Table Prefix:</th>
                    <td><input name="mpp_mint_prefix" type="text" id="mpp_mint_prefix" value="'.$mpp_mint_prefix.'" size="50" />
                    <br />default: mint_</td>
               </tr>
               <tr valign="top">
                    <th scope="row">URL Ignore List:</th>
                    <td><input name="mpp_mint_ign_list" type="text" id="mpp_mint_ign_list" value="'.$mpp_mint_ign_list.'" size="100" />
                    <br />comma separated list of URLs to ignore</td>
               </tr>
               <tr valign="top">
                    <th scope="row">Extensions Ignore List:</th>
                    <td><input name="mpp_mint_ign_ext" type="text" id="mpp_mint_ign_ext" value="'.$mpp_mint_ign_ext.'" size="100" />
                    <br />comma separated list of file extensions to ignore</td>
               </tr>
               <tr valign="top">
                    <th scope="row">Required URL Pattern:</th>
                    <td><input name="mpp_mint_url_pat" type="text" id="mpp_mint_url_pat" value="'.$mpp_mint_url_pat.'" size="100" />
                    <br />comma separated list of URL patterns that must be matched (use % as placeholder for any sequence of characters e. g. http://turmsegler.net/200%)</td>
               </tr>
               <tr valign="top">
                    <th scope="row">String to Remove from Post Titles:</th>
                    <td><input name="mpp_mint_title_rem" type="text" id="mpp_mint_title_rem" value="'.$mpp_mint_title_rem.'" size="100" />
                    <br />e. g. Your_Blog_name &laquo; </td>
               </tr>
               </table>
    
               <h3>Sample - 5 Most Popluar Posts</h3>
               <ul>';
    echo mpp_popular_posts( 5 );
    echo      '</ul>
               <h3>Sample - Top 5 from Mint Watchlist</h3>
               <ul>';
    echo mpp_watched_posts( 5 );
    echo      '</ul>
        <p class="submit">
            <input type="submit" name="Submit" value="Update Options &raquo;" />
        </p>
        </form>
    
    </div>';
}

function mmp_unserialize($serialized)
{
    $serialized    = stripslashes($serialized);
    $unserialized  = @unserialize($serialized);

    if ($unserialized === false)
    {
        $serialized   = preg_replace('/s:\d+:"([^"]*)";/e', "'s:'.strlen('\\1').':\"\\1\";'", $serialized);
        $unserialized = @unserialize(stripslashes($serialized));
    }
    
    return $unserialized;
}

function mpp_add_options_panel()
{
    if(function_exists("add_options_page"))
        add_options_page("MintPopularPostsWP", "Mint Posts", 5, basename(__FILE__), "mpp_options_panel");
}

if(function_exists("add_action"))
    add_action("admin_menu"  , "mpp_add_options_panel");

?>